@@ -335,15 +335,24 @@ class WeightsStrategy:
 # dl
 
 
-def empty_cuda_cache(device: Union[int, str, torch.device]) -> None:
+def get_torch_device(device: Union[int, str, torch.device]) -> torch.device:
     if isinstance(device, (int, str)):
         device = torch.device(device)
+    return device
+
+
+def empty_cuda_cache(device: Union[int, str, torch.device]) -> None:
+    device = get_torch_device(device)
     if device.type != "cuda":
         return
     with torch.cuda.device(device):
         torch.cuda.empty_cache()
 
 
+def is_cpu(device: Union[int, str, torch.device]) -> bool:
+    return get_torch_device(device).type == "cpu"
+
+
 def safe_clip_(net: Tensor) -> None:
     finfo = torch.finfo(net.dtype)
     net.clamp_(finfo.min, finfo.max)
